import 'package:flutter/material.dart';
import 'package:flutter_bloc/flutter_bloc.dart';
import 'package:ui_sdk/utils/pubsub/localpubsub.dart';
import 'package:ui_sdk/props/StandardScreenResponse.dart';
import 'ApiResponseState.dart';
import 'Renderer.dart';
import 'utils/init_data.dart';

abstract class StandardPage {
  late StatefulWidget page;

  dynamic getData();
  Set<String?> getActions();
  dynamic getPageCallBackForAction();
  dynamic getPageContext();
  dynamic getConstructorArgs();

  StandardPage([InitData? args]) {
    page = FlutterPage(
      apiData: getData(),
      actionList: getActions(),
      fun: getPageCallBackForAction(),
      reference: getPageContext(),
      constructorArgs: getConstructorArgs(),
      args: args,
    );
  }

  StatefulWidget getPage() {
    return page;
  }
}

class FlutterPage extends StatefulWidget {
  final dynamic apiData;
  final Set<String?> actionList;
  final dynamic fun;
  final dynamic reference;
  Map<String?, Subscription?>? subArray;
  final dynamic constructorArgs;
  final InitData? args;

  FlutterPage({
    required this.apiData,
    required this.actionList,
    required this.fun,
    required this.reference,
    required this.constructorArgs,
    Key? key,
    this.args,
  }) : super(key: key);

  @override
  State<FlutterPage> createState() => FlutterPageState();
}

class FlutterPageState extends State<FlutterPage> {
  final Renderer renderer = Renderer();

  @override
  void initState() {
    super.initState();
    registerEvents();
  }

  @override
  void didChangeDependencies() {
    super.didChangeDependencies();
    final dynamic args = ModalRoute.of(context)!.settings.arguments ?? {};
    final dynamic constructorArguments = widget.constructorArgs;
    final Map<dynamic, dynamic> finalArgs = {...constructorArguments, ...args};
    final apiCubit = BlocProvider.of<ApiResponseCubit>(context);
    apiCubit.fetchData(widget.apiData, finalArgs, widget.reference);
  }

  void registerEvents() {
    widget.subArray = Map(); //string,Subscription
    final PubSub pubsub = PubSub();

    widget.actionList.forEach((element) {
      Subscription? sub;
      sub = pubsub.subscribe(element, widget.reference);
      widget.subArray![element] = sub;

      sub?.stream?.listen((uikAction) {
        if (widget.reference == pubsub.stack.top()) {
          widget.fun(uikAction);
        }
      });
    });

    pubsub.stack.push(widget.reference);
  }

  Future<void> _handleRefresh() async {
    final apiCubit = BlocProvider.of<ApiResponseCubit>(context);
    final dynamic args = ModalRoute.of(context)!.settings.arguments ?? {};
    final dynamic constructorArguments = widget.constructorArgs;
    final Map<dynamic, dynamic> finalArgs = {...constructorArguments, ...args};
    apiCubit.fetchData(widget.apiData, finalArgs, widget.reference);
  }

  @override
  Widget build(BuildContext context) {
    final apiCubit = BlocProvider.of<ApiResponseCubit>(context);
    final standardScreenResponseCubit = BlocProvider.of<StandardScreenResponseCubit>(context);

    return WillPopScope(
      onWillPop: () async {
        final PubSub pubsub = PubSub();
        pubsub.stack.pop();
        pubsub.unsubscribe(widget.subArray);
        return true;
      },
      child: Scaffold(
        body: BlocBuilder<ApiResponseCubit, ApiResponseState>(
          builder: (context, apiState) {
            return BlocBuilder<StandardScreenResponseCubit, Map<String, StandardScreenResponse?>>(
              builder: (context, standardScreenResponseMap) {
                if (apiState.isPullToRefreshEnabled) {
                  return RefreshIndicator(
                    color: Colors.yellow,
                    onRefresh: _handleRefresh,
                    child: _buildContent(context, apiState, standardScreenResponseMap),
                  );
                } else {
                  return _buildContent(context, apiState, standardScreenResponseMap);
                }
              },
            );
          },
        ),
      ),
    );
  }

  Widget _buildContent(BuildContext context, ApiResponseState apiState, Map<String, StandardScreenResponse?> standardScreenResponseMap) {
    if (apiState.responseState == RequestState.loading) {
      return Center(
        child: CircularProgressIndicator(
          color: Colors.yellow,
        ),
      );
    } else if (apiState.responseState == RequestState.loaded) {
      final response = apiState.apiResponse;
      if (response != null && response.isSuccess == true) {
        final standardScreenResponse = standardScreenResponseMap[widget.reference];
        if (standardScreenResponse != null) {
          return renderer.getListWidgetUi(
            standardScreenResponse,
            context,
          );
        } else {
          return Text("Standard screen response is null.");
        }
      } else {
        return Text("Error: ${response?.error}");
      }
    } else {
      return Text("Failed to load data.");
    }
  }

}
